package org.jtheque.filmstobuy.persistence.dao.impl;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.core.managers.persistence.Entity;
import org.jtheque.core.managers.persistence.GenericDao;
import org.jtheque.core.managers.persistence.Query;
import org.jtheque.core.managers.persistence.QueryMapper;
import org.jtheque.core.managers.persistence.context.IDaoPersistenceContext;
import org.jtheque.utils.bean.IntDate;
import org.jtheque.filmstobuy.persistence.dao.able.IDaoFilmsToBuy;
import org.jtheque.filmstobuy.persistence.od.FilmToBuyImpl;
import org.jtheque.filmstobuy.persistence.od.abstraction.FilmToBuy;
import org.springframework.jdbc.core.simple.ParameterizedRowMapper;

import javax.annotation.Resource;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

/**
 * Data Access Object implementation for films to buy.
 *
 * @author Baptiste Wicht
 */
public final class DaoFilmsToBuy extends GenericDao<FilmToBuyImpl> implements IDaoFilmsToBuy {
    private final ParameterizedRowMapper<FilmToBuyImpl> rowMapper = new FilmToBuyRowMapper();
    private final QueryMapper queryMapper = new FilmToBuyQueryMapper();

    @Resource
    private IDaoPersistenceContext persistenceContext;

    /**
     * Construct a new DaoFilmsToBuy.
     */
    public DaoFilmsToBuy() {
        super(TABLE);
    }

    @Override
    public List<FilmToBuyImpl> getFilmsToBuy() {
        return getAll();
    }

    @Override
    public FilmToBuyImpl getFilmToBuy(int id) {
        return get(id);
    }

    @Override
    protected ParameterizedRowMapper<FilmToBuyImpl> getRowMapper() {
        return rowMapper;
    }

    @Override
    protected QueryMapper getQueryMapper() {
        return queryMapper;
    }

    @Override
    protected void loadCache() {
        List<FilmToBuyImpl> filmToBuys = persistenceContext.getSortedList(TABLE, rowMapper);

        for (FilmToBuyImpl filmToBuy : filmToBuys) {
            getCache().put(filmToBuy.getId(), filmToBuy);
        }

        setCacheEntirelyLoaded(true);
    }

    @Override
    protected void load(int i) {
        FilmToBuyImpl filmToBuy = persistenceContext.getDataByID(TABLE, i, rowMapper);

        getCache().put(i, filmToBuy);
    }

    /**
     * A row mapper to map ResultSet to FilmToBuyImpl.
     *
     * @author Baptiste Wicht
     */
    private static class FilmToBuyRowMapper implements ParameterizedRowMapper<FilmToBuyImpl> {
        @Override
        public FilmToBuyImpl mapRow(ResultSet rs, int i) throws SQLException {
            FilmToBuyImpl filmToBuy = new FilmToBuyImpl();

            filmToBuy.setId(rs.getInt("ID"));
            filmToBuy.setTitle(rs.getString("TITLE"));
            filmToBuy.setDate(new IntDate(rs.getInt("ADD_DATE")));

            return filmToBuy;
        }
    }

    /**
     * A row mapper to map a FilmToBuyImpl to a request.
     *
     * @author Baptiste Wicht
     */
    private static class FilmToBuyQueryMapper implements QueryMapper {
        @Override
        public Query constructInsertQuery(Entity entity) {
            FilmToBuy filmToBuy = (FilmToBuy)entity;

            String query = "INSERT INTO " + IDaoFilmsToBuy.TABLE + " (TITLE, ADD_DATE) VALUES(?, ?)";

            Object[] parameters = {
                    filmToBuy.getTitle(),
                    filmToBuy.getDate().intValue()
            };

            return new Query(query, parameters);
        }

        @Override
        public Query constructUpdateQuery(Entity entity) {
            FilmToBuy filmToBuy = (FilmToBuy)entity;

            String query = "UPDATE " + IDaoFilmsToBuy.TABLE + " SET TITLE = ?, ADD_DATE = ? WHERE ID = ?";

            Object[] parameters = {
                    filmToBuy.getTitle(),
                    filmToBuy.getDate().intValue(),
                    filmToBuy.getId()
            };

            return new Query(query, parameters);
        }
    }
}